/** @file doxyref.h
 * Doxygen documentation file. This file generates Doxygen
 * documentation pages from files in the C64x+ AES library source
 * tree.
 *
 * @version 1.00
 */

/** \mainpage C64x+ AES library
\section intro User Documentation for the C64x+ AES library
This is the user documentation for the C64x+ AES library.  It can be
automatically generated with the doxygen utility (see
http://www.doxygen.org).  For best results, please enable
JAVA_AUTOBRIEF mode (see the file Doxyfile for sample doxygen settings).

The C64x+ AES source code distribution consists of two main items:

- \subpage libsource
- \subpage testsource

The provided Makefile will compile both items, creating the files
\e AES64plus.lib and \e test.out respectively.

\verbatim
>gmake
cl6x -q -pdr -o2 -ms0 -mv6400+ aes128_keyschedule.sa
cl6x -q -pdr -o2 -ms0 -mv6400+ aes192_keyschedule.sa
cl6x -q -pdr -o2 -ms0 -mv6400+ aes256_keyschedule.sa
cl6x -q -pdr -o2 -ms0 -mv6400+ aes_decrypt.sa
cl6x -q -pdr -o2 -ms0 -mv6400+ aes_encrypt.sa
cl6x -q -pdr -o2 -ms0 -mv6400+ aes_data.asm
ar6x ru AES64plus.lib aes128_keyschedule.obj aes192_keyschedule.obj aes256_keyschedule.obj aes_decrypt.obj aes_encrypt.obj aes_data.obj
  ==>  building archive 'AES64plus.lib'
cl6x -q -pdr -o2 -ms0 -mv6400+ test.c
cl6x -q -pdr -o2 -ms0 -mv6400+ AESAVS-128.c
cl6x -q -pdr -o2 -ms0 -mv6400+ AESAVS-192.c
cl6x -q -pdr -o2 -ms0 -mv6400+ AESAVS-256.c
cl6x -z -c -m test.map -o test.out -w -x test.cmd test.obj AESAVS-128.obj AESAVS-192.obj AESAVS-256.obj -l=AES64plus.lib -l=rts64plus.lib"
<Linking>
All done.
\endverbatim

\section copyright Copyright
Copyright (C) 2004-2010 Texas Instruments Incorporated - http://www.ti.com/ 

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the 
   documentation and/or other materials provided with the   
   distribution.

 - Neither the name of Texas Instruments Incorporated nor the names of
   its contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

\section export_info Export Control Information
This distribution includes cryptographic software.  The country in
which you currently reside may have restrictions on the import,
possession, use, and/or re-export to another country, of encryption
software.  BEFORE using any encryption software, please check your
country's laws, regulations and policies concerning the import,
possession, or use, and re-export of encryption software, to see if
this is permitted.
 
The U.S. Government Department of Commerce, Bureau of Industry and
Security (BIS), has classified this software as Export Commodity
Control Number (ECCN) 5D002.C.1, which includes information security
software using or performing cryptographic functions with symmetric
algorithms.  The form and manner of this distribution makes it
eligible for export under the License Exception ENC Technology
Software Unrestricted (TSU) exception (see the BIS Export
Administration Regulations, Section 740.13) for both object code and
source code.
*/

/** \page libsource C64x+ AES library source code

The C64x+ AES library source code implements the Advanced Encryption
Standard (AES) algorithm for all three legal key sizes (128, 192, and
256 bits). Only the basic ECB mode is provided; other operating modes
can be easily added through wrapper functions.

The library source code consists of the following source files:

- aes.h - AES function prototypes and structure information
- aes128_keyschedule.sa - key schedule generation code for 128 bit keys
- aes192_keyschedule.sa - key schedule generation code for 192 bit keys
- aes256_keyschedule.sa - key schedule generation code for 256 bit keys
- aes_data.asm - Sbox data tables
- aes_decrypt.sa - AES ECB mode decryption code for all key sizes
- aes_encrypt.sa - AES ECB mode encryption code for all key sizes

\section AES_lib_usage AES library usage
To use AES routines in a C language application, include the file
aes.h, and link against the AES64plus.lib library.

\code
#include "aes.h"
\endcode

To perform an encryption operation with a 128-bit key:
\code
    AES128_KeySchedule encSchedule, decSchedule;
    uint8_t key[AES128_KEYSIZE];

      . . . . .

    AES128_keySchedule((pAES128_Key)key, &encSchedule, &decSchedule);
    AES_encrypt(pPlainText, pCodeText, &encSchedule, AES128_ROUNDS);
\endcode

The variable \e key should be set to the correct 128 bit key value
before the call to AES128_keySchedule().  The parameters \e pPlainText
and \e pCodeText should point to the plaintext to be encrypted and the
buffer to receive the encrypted result respectively.  Note that these
pointers may be the same, in which case the encrypted result will
overwrite the original plaintext (encryption in place).

Similarly, decryption with a 192 bit key value can be performed as follows:
\code
    AES192_KeySchedule encSchedule, decSchedule;
    uint8_t key[AES192_KEYSIZE];

      . . . . .

    AES192_keySchedule((pAES192_Key)key, &encSchedule, &decSchedule);
    AES_decrypt(pCodeText, pPlainText, &decSchedule, AES192_ROUNDS);
\endcode

And finally, encryption with a 256-bit key:
\code
    AES256_KeySchedule encSchedule, decSchedule;
    uint8_t key[AES256_KEYSIZE];

      . . . . .

    AES128_keySchedule((pAES256_Key)key, &encSchedule, &decSchedule);
    AES_encrypt(pPlainText, pCodeText, &encSchedule, AES256_ROUNDS);
\endcode

For more examples, please refer to the file test.c.

\section performance Performance Data
The following table shows the performance data for the different key
sizes.  The data is expressed in processor clock cycles, and includes
the call and return overhead.

<table border>
<tr> <td> Key Size </td> <td> Encrypt </td> <td> Decrypt </td> <td> Schedule </td> </tr>
<tr> <td> 128 </td> <td> 195 </td> <td> 192 </td> <td> 189 </td> </tr>
<tr> <td> 192 </td> <td> 227 </td> <td> 229 </td> <td> 281 </td> </tr>
<tr> <td> 256 </td> <td> 259 </td> <td> 261 </td> <td> 351 </td> </tr>
</table>

*/

/** \page testsource AES Algorithm Validation Suite Test Harness
The AES Algorithm Validation Suite AESAVS Test Harness source code
consists of the following source files:

- test.c - AESAVS Test Harness code
- test.cmd - linker command file for DM6446 EVM
- testVectors.h - structure definition for various test vectors
- AESAVS-128.c - test vectors for 128 bit keys
- AESAVS-192.c - test vectors for 192 bit keys
- AESAVS-256.c - test vectors for 256 bit keys

The linker command file (test.cmd) and the board specific values in
test.c (\ref L2CFG_REG and \ref MAR_EXTMEM) are configured for a
DaVinci EVM board.  These defines and the linker command file may have
to be changed if you want to run the test suite on different hardware.

Executing the generated test.out file should provide the following output:
\verbatim
NIST AES Algorithm Validation Suite
Electronic Code Book (ECB) Mode
Key size = 128 bits

Known Answer Tests
  GFSbox encrypt: Passed
  GFSbox decrypt: Passed
  KeySbox encrypt: Passed
  KeySbox decrypt: Passed
  Variable Text encrypt: Passed
  Variable Text decrypt: Passed
  Variable Key encrypt: Passed
  Variable Key decrypt: Passed
Monte Carlo Tests
  Encryption Tests: Passed
  Decryption Tests: Passed

Key size = 192 bits

Known Answer Tests
  GFSbox encrypt: Passed
  GFSbox decrypt: Passed
  KeySbox encrypt: Passed
  KeySbox decrypt: Passed
  Variable Text encrypt: Passed
  Variable Text decrypt: Passed
  Variable Key encrypt: Passed
  Variable Key decrypt: Passed
Monte Carlo Tests
  Encryption Tests: Passed
  Decryption Tests: Passed

Key size = 256 bits

Known Answer Tests
  GFSbox encrypt: Passed
  GFSbox decrypt: Passed
  KeySbox encrypt: Passed
  KeySbox decrypt: Passed
  Variable Text encrypt: Passed
  Variable Text decrypt: Passed
  Variable Key encrypt: Passed
  Variable Key decrypt: Passed
Monte Carlo Tests
  Encryption Tests: Passed
  Decryption Tests: Passed
\endverbatim
*/
